@@ -1,7 +1,8 @@
+#!/usr/bin/env python3
 """Version information for PCILeech Firmware Generator."""
 
-__version__ = "0.5.8"
-__version_info__ = (0, 5, 7)
+__version__ = "0.13.34"
+__version_info__ = (0, 13, 34)
 
 # Release information
 __title__ = "PCILeech Firmware Generator"
@@ -9,8 +10,8 @@ __description__ = "Generate spoofed PCIe DMA firmware from real donor hardware"
 __author__ = "Ramsey McGrath"
 __author_email__ = "ramsey@voltcyclone.info"
 __license__ = "MIT"
-__url__ = "https://github.com/ramseymcgrath/PCILeechFWGenerator"
+__url__ = "https://github.com/voltcyclone/PCILeechFWGenerator"
 
 # Build metadata
-__build_date__ = "2025-06-05T01:44:07.732192"
-__commit_hash__ = "39c13da"
+__build_date__ = "2025-09-25T21:06:22.332950"
+__commit_hash__ = "4fedfb8"
