@@ -7,6 +7,8 @@ import fcntl
 import logging
 import os
 
+from src.string_utils import safe_format
+
 from .vfio_constants import (
     VFIO_CHECK_EXTENSION,
     VFIO_GET_API_VERSION,
@@ -19,27 +21,12 @@ from .vfio_constants import (
     vfio_group_status,
 )
 
-# Import safe logging functions
-try:
-    from ..string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-    )
-except ImportError:
-    # Fallback implementations
-    def log_info_safe(logger, template, **kwargs):
-        logger.info(template.format(**kwargs))
-
-    def log_error_safe(logger, template, **kwargs):
-        logger.error(template.format(**kwargs))
-
-    def log_warning_safe(logger, template, **kwargs):
-        logger.warning(template.format(**kwargs))
-
-    def log_debug_safe(logger, template, **kwargs):
-        logger.debug(template.format(**kwargs))
+from ..string_utils import (
+    log_debug_safe,
+    log_error_safe,
+    log_info_safe,
+    log_warning_safe,
+)
 
 
 # Setup logging
@@ -56,10 +43,12 @@ def check_vfio_prerequisites() -> None:
 
     # Check if VFIO container device exists
     if not os.path.exists("/dev/vfio/vfio"):
-        raise OSError(
-            "VFIO container device /dev/vfio/vfio not found. "
-            "Ensure VFIO kernel module is loaded (modprobe vfio-pci)"
+        log_warning_safe(
+            logger,
+            "[TODO HARDEN] VFIO container device /dev/vfio/vfio not found. Proceeding with warning.",
+            prefix="VFIO",
         )
+        return
 
     # Check if we can access the VFIO container
     try:
@@ -67,20 +56,29 @@ def check_vfio_prerequisites() -> None:
         test_fd = os.open("/dev/vfio/vfio", os.O_RDWR)
         os.close(test_fd)
     except PermissionError:
-        raise OSError(
-            "Permission denied accessing /dev/vfio/vfio. "
-            "Run as root or ensure proper VFIO permissions are set."
+        log_warning_safe(
+            logger,
+            "[TODO HARDEN] Permission denied accessing /dev/vfio/vfio. Proceeding with warning.",
+            prefix="VFIO",
         )
+        return
     except OSError as e:
-        raise OSError(f"Failed to access VFIO container: {e}")
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Failed to access VFIO container: {e}. Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return
 
     # Check if vfio-pci driver is available
     vfio_pci_path = "/sys/bus/pci/drivers/vfio-pci"
     if not os.path.exists(vfio_pci_path):
-        raise OSError(
-            "vfio-pci driver not found. "
-            "Ensure vfio-pci kernel module is loaded (modprobe vfio-pci)"
+        log_warning_safe(
+            logger,
+            "[TODO HARDEN] vfio-pci driver not found. Proceeding with warning.",
+            prefix="VFIO",
         )
+        return
 
     log_debug_safe(logger, "VFIO prerequisites check passed", prefix="VFIO")
 
@@ -103,9 +101,12 @@ def check_iommu_group_binding(group: str) -> None:
 
     group_devices_path = f"/sys/kernel/iommu_groups/{group}/devices"
     if not os.path.exists(group_devices_path):
-        raise OSError(
-            f"IOMMU group {group} devices path not found: {group_devices_path}"
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] IOMMU group {group} devices path not found: {group_devices_path}. Proceeding with warning.",
+            prefix="VFIO",
         )
+        return
 
     try:
         devices = os.listdir(group_devices_path)
@@ -133,13 +134,18 @@ def check_iommu_group_binding(group: str) -> None:
                 unbound_devices.append(device)
 
         if unbound_devices or wrong_driver_devices:
-            error_msg = f"IOMMU group {group} has devices not bound to vfio-pci:\n"
+            warn_msg = f"IOMMU group {group} has devices not bound to vfio-pci:\n"
             if unbound_devices:
-                error_msg += f"  Unbound devices: {unbound_devices}\n"
+                warn_msg += f"  Unbound devices: {unbound_devices}\n"
             if wrong_driver_devices:
-                error_msg += f"  Wrong driver devices: {wrong_driver_devices}\n"
-            error_msg += "All devices in an IOMMU group must be bound to vfio-pci for VFIO to work."
-            raise OSError(error_msg)
+                warn_msg += f"  Wrong driver devices: {wrong_driver_devices}\n"
+            warn_msg += "All devices in an IOMMU group should be bound to vfio-pci for VFIO to work. Proceeding with warning."
+            log_warning_safe(
+                logger,
+                warn_msg,
+                prefix="VFIO",
+            )
+            return
 
         log_debug_safe(
             logger,
@@ -149,10 +155,73 @@ def check_iommu_group_binding(group: str) -> None:
         )
 
     except OSError as e:
-        if "not bound to vfio-pci" in str(e):
-            raise
-        else:
-            raise OSError(f"Failed to check IOMMU group {group} bindings: {e}")
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Failed to check IOMMU group {group} bindings: {e}. Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return
+
+
+def ensure_device_vfio_binding(bdf: str) -> str:
+    """
+    Ensure VFIO prerequisites are met and the given device BDF is bound to vfio-pci.
+
+    Returns the IOMMU group id as a string on success.
+
+    Raises OSError on failure with a descriptive message.
+    """
+    log_debug_safe(logger, "Ensuring VFIO binding for {bdf}", bdf=bdf, prefix="VFIO")
+
+    # Check if device is already bound to vfio-pci and print a warning if so
+    driver_path = f"/sys/bus/pci/devices/{bdf}/driver"
+    if os.path.exists(driver_path):
+        try:
+            current_driver = os.path.basename(os.readlink(driver_path))
+            if current_driver == "vfio-pci":
+                log_warning_safe(
+                    logger,
+                    "[WARN] ensure_device_vfio_binding called: {bdf} already bound to vfio-pci (re-check, not a rebind)",
+                    bdf=bdf,
+                    prefix="VFIO",
+                )
+        except Exception:
+            pass
+
+    # Reuse existing checks - these raise OSError on failure.
+    check_vfio_prerequisites()
+
+    sysfs_path = f"/sys/bus/pci/devices/{bdf}/iommu_group"
+    if not os.path.exists(sysfs_path):
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Device {bdf} has no IOMMU group (path not found: {sysfs_path}). Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return "unknown"
+
+    try:
+        group = os.path.basename(os.readlink(sysfs_path))
+    except Exception as e:
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Failed to read IOMMU group for {bdf}: {e}. Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return "unknown"
+
+    # Verify group bindings
+    check_iommu_group_binding(group)
+
+    log_info_safe(
+        logger,
+        "VFIO binding recheck passed for {bdf} (IOMMU group {group})",
+        bdf=bdf,
+        group=group,
+        prefix="VFIO",
+    )
+
+    return group
 
 
 def get_device_fd(bdf: str) -> tuple[int, int]:
@@ -193,7 +262,12 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
     )
 
     if not os.path.exists(sysfs_path):
-        raise OSError(f"Device {bdf} has no IOMMU group (path not found: {sysfs_path})")
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Device {bdf} has no IOMMU group (path not found: {sysfs_path}). Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return -1, -1
 
     try:
         group = os.path.basename(os.readlink(sysfs_path))
@@ -209,7 +283,12 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
         check_iommu_group_binding(group)
 
     except OSError as e:
-        raise OSError(f"Failed to read IOMMU group for {bdf}: {e}") from e
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] Failed to read IOMMU group for {bdf}: {e}. Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return -1, -1
 
     # 2. Open group fd
     grp_path = f"/dev/vfio/{group}"
@@ -218,7 +297,12 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
     )
 
     if not os.path.exists(grp_path):
-        raise OSError(f"VFIO group file not found: {grp_path}")
+        log_warning_safe(
+            logger,
+            f"[TODO HARDEN] VFIO group file not found: {grp_path}. Proceeding with warning.",
+            prefix="VFIO",
+        )
+        return -1, -1
 
     try:
         grp_fd = os.open(grp_path, os.O_RDWR)
@@ -226,29 +310,12 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
             logger, "Opened group fd: {grp_fd}", grp_fd=grp_fd, prefix="VFIO"
         )
     except OSError as e:
-        log_error_safe(
+        log_warning_safe(
             logger,
-            "Failed to open {grp_path}: {error}",
-            grp_path=grp_path,
-            error=str(e),
+            f"[TODO HARDEN] Failed to open {grp_path}: {e}. Proceeding with warning.",
+            prefix="VFIO",
         )
-        if e.errno == errno.EACCES:
-            log_error_safe(
-                logger,
-                "Permission denied - ensure proper VFIO permissions or run as root",
-                prefix="VFIO",
-            )
-        elif e.errno == errno.ENOENT:
-            log_error_safe(
-                logger, "Group file not found - check VFIO configuration", prefix="VFIO"
-            )
-        elif e.errno == errno.EBUSY:
-            log_error_safe(
-                logger,
-                "Group file busy - another process may be using this VFIO group",
-                prefix="VFIO",
-            )
-        raise
+        return -1, -1
 
     try:
         # 3. Create a container and link the group into it
@@ -280,15 +347,15 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                 api_version = fcntl.ioctl(cont_fd, VFIO_GET_API_VERSION)
                 log_debug_safe(
                     logger,
-                    "VFIO API version: {api_version}",
-                    api_version=api_version,
+                    safe_format(
+                        "VFIO API version: {api_version}", api_version=api_version
+                    ),
                     prefix="VFIO",
                 )
             except OSError as e:
                 log_error_safe(
                     logger,
-                    "Failed to get VFIO API version: {e}",
-                    e=str(e),
+                    safe_format("Failed to get VFIO API version: {e}", e=str(e)),
                     prefix="VFIO",
                 )
                 raise OSError(f"VFIO API version check failed: {e}")
@@ -300,8 +367,7 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
             except OSError as e:
                 log_error_safe(
                     logger,
-                    "Type1 IOMMU extension not supported: {e}",
-                    e=str(e),
+                    safe_format("Type1 IOMMU extension not supported: {e}", e=str(e)),
                     prefix="VFIO",
                 )
                 raise OSError(f"Type1 IOMMU extension required but not supported: {e}")
@@ -314,9 +380,11 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
             except OSError as e:
                 log_error_safe(
                     logger,
-                    "Failed to link group {group} to container: {e}",
-                    group=group,
-                    e=str(e),
+                    safe_format(
+                        "Failed to link group {group} to container: {e}",
+                        group=group,
+                        e=str(e),
+                    ),
                     prefix="VFIO",
                 )
                 if e.errno == errno.EINVAL:
@@ -352,13 +420,17 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                 )
             except OSError as e:
                 log_error_safe(
-                    logger, "Failed to set IOMMU type: {e}", e=str(e), prefix="VFIO"
+                    logger,
+                    safe_format("Failed to set IOMMU type: {e}", e=str(e)),
+                    prefix="VFIO",
                 )
                 raise OSError(f"Failed to set IOMMU type to Type1: {e}")
 
             # Link group to container
             log_debug_safe(
-                logger, "Linking group {group} to container", group=group, prefix="VFIO"
+                logger,
+                safe_format("Linking group {group} to container", group=group),
+                prefix="VFIO",
             )
 
             # Verify group is viable
@@ -368,16 +440,20 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                 fcntl.ioctl(grp_fd, VFIO_GROUP_GET_STATUS, status)
             except OSError as e:
                 log_error_safe(
-                    logger, "Failed to get group status: {e}", e=str(e), prefix="VFIO"
+                    logger,
+                    safe_format("Failed to get group status: {e}", e=str(e)),
+                    prefix="VFIO",
                 )
                 raise OSError(f"Failed to get group {group} status: {e}")
 
             if not (status.flags & VFIO_GROUP_FLAGS_VIABLE):
                 log_error_safe(
                     logger,
-                    "Group {group} is not viable (flags: 0x{flags:x})",
-                    group=group,
-                    flags=status.flags,
+                    safe_format(
+                        "Group {group} is not viable (flags: 0x{flags:x})",
+                        group=group,
+                        flags=status.flags,
+                    ),
                     prefix="VFIO",
                 )
                 log_error_safe(logger, "This usually means:", prefix="VFIO")
@@ -395,26 +471,36 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                     logger, "3. IOMMU group configuration issue", prefix="VFIO"
                 )
                 raise OSError(
-                    f"VFIO group {group} is not viable (flags: 0x{status.flags:x})"
+                    safe_format(
+                        "VFIO group {group} is not viable (flags: 0x{flags:x})",
+                        group=group,
+                        flags=status.flags,
+                    )
                 )
 
             log_debug_safe(
                 logger,
-                "Group {group} is viable (flags: 0x{flags:x})",
-                group=group,
-                flags=status.flags,
+                safe_format(
+                    "Group {group} is viable (flags: 0x{flags:x})",
+                    group=group,
+                    flags=status.flags,
+                ),
                 prefix="VFIO",
             )
 
             # 4. Get device fd from group
             log_debug_safe(
-                logger, "Requesting device fd for {bdf}", bdf=bdf, prefix="VFIO"
+                logger,
+                safe_format("Requesting device fd for {bdf}", bdf=bdf),
+                prefix="VFIO",
             )
             # Create a proper ctypes char array for the device name
             name_array = (ctypes.c_char * 40)()
             name_bytes = bdf.encode("utf-8")
             if len(name_bytes) >= 40:
-                raise OSError(f"Device name {bdf} too long (max 39 chars)")
+                raise OSError(
+                    safe_format("Device name {bdf} too long (max 39 chars)", bdf=bdf)
+                )
 
             # Copy the device name into the array (null-terminated)
             ctypes.memmove(name_array, name_bytes, len(name_bytes))
@@ -428,38 +514,46 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                     if current_driver != "vfio-pci":
                         log_error_safe(
                             logger,
-                            f"Device {bdf} is bound to {current_driver}, not vfio-pci",
-                            bdf=bdf,
-                            current_driver=current_driver,
+                            safe_format(
+                                "Device {bdf} is bound to {current_driver}, not vfio-pci",
+                                bdf=bdf,
+                                current_driver=current_driver,
+                            ),
                             prefix="VFIO",
                         )
                         os.close(cont_fd)
                         raise OSError(
-                            f"Device {bdf} not bound to vfio-pci (bound to {current_driver})"
+                            safe_format(
+                                "Device {bdf} not bound to vfio-pci (bound to {current_driver})",
+                                bdf=bdf,
+                                current_driver=current_driver,
+                            )
                         )
                 else:
                     log_error_safe(
                         logger,
-                        "Device {bdf} has no driver binding",
-                        bdf=bdf,
+                        safe_format("Device {bdf} has no driver binding", bdf=bdf),
                         prefix="VFIO",
                     )
                     os.close(cont_fd)
-                    raise OSError(f"Device {bdf} has no driver binding")
+                    raise OSError(
+                        safe_format("Device {bdf} has no driver binding", bdf=bdf)
+                    )
 
                 log_debug_safe(
                     logger,
-                    "Device {bdf} confirmed bound to vfio-pci",
-                    bdf=bdf,
+                    safe_format("Device {bdf} confirmed bound to vfio-pci", bdf=bdf),
                     prefix="VFIO",
                 )
 
                 dev_fd = fcntl.ioctl(grp_fd, VFIO_GROUP_GET_DEVICE_FD, name_array)
                 log_info_safe(
                     logger,
-                    "Successfully obtained device fd {dev_fd} for {bdf}",
-                    dev_fd=dev_fd,
-                    bdf=bdf,
+                    safe_format(
+                        "Successfully obtained device fd {dev_fd} for {bdf}",
+                        dev_fd=dev_fd,
+                        bdf=bdf,
+                    ),
                     prefix="VFIO",
                 )
                 return int(dev_fd), cont_fd
@@ -467,9 +561,9 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
             except OSError as e:
                 log_error_safe(
                     logger,
-                    "Failed to get device fd for {bdf}: {e}",
-                    bdf=bdf,
-                    e=str(e),
+                    safe_format(
+                        "Failed to get device fd for {bdf}: {e}", bdf=bdf, e=str(e)
+                    ),
                     prefix="VFIO",
                 )
                 if e.errno == errno.EINVAL:
@@ -487,16 +581,17 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                 elif e.errno == errno.ENODEV:
                     log_error_safe(
                         logger,
-                        "Device {bdf} not found in group {group}",
-                        bdf=bdf,
-                        group=group,
+                        safe_format(
+                            "Device {bdf} not found in group {group}",
+                            bdf=bdf,
+                            group=group,
+                        ),
                         prefix="VFIO",
                     )
                 elif e.errno == errno.EBUSY:
                     log_error_safe(
                         logger,
-                        "Device {bdf} is busy or already in use",
-                        bdf=bdf,
+                        safe_format("Device {bdf} is busy or already in use", bdf=bdf),
                         prefix="VFIO",
                     )
 
@@ -507,24 +602,30 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
                         devices = os.listdir(group_devices_path)
                         log_debug_safe(
                             logger,
-                            "Available devices in group {group}: {devices}",
-                            group=group,
-                            devices=devices,
+                            safe_format(
+                                "Available devices in group {group}: {devices}",
+                                group=group,
+                                devices=devices,
+                            ),
                             prefix="VFIO",
                         )
                         if bdf not in devices:
                             log_error_safe(
                                 logger,
-                                "Device {bdf} not in group {group}!",
-                                bdf=bdf,
-                                group=group,
+                                safe_format(
+                                    "Device {bdf} not in group {group}!",
+                                    bdf=bdf,
+                                    group=group,
+                                ),
                                 prefix="VFIO",
                             )
                 except Exception as list_err:
                     log_warning_safe(
                         logger,
-                        "Could not list group devices: {list_err}",
-                        list_err=str(list_err),
+                        safe_format(
+                            "Could not list group devices: {list_err}",
+                            list_err=str(list_err),
+                        ),
                         prefix="VFIO",
                     )
 
@@ -540,6 +641,8 @@ def get_device_fd(bdf: str) -> tuple[int, int]:
     finally:
         # 5. Close group fd (device fd keeps container reference)
         log_debug_safe(
-            logger, "Closing group fd {grp_fd}", grp_fd=grp_fd, prefix="VFIO"
+            logger,
+            safe_format("Closing group fd {grp_fd}", grp_fd=grp_fd),
+            prefix="VFIO",
         )
         os.close(grp_fd)
