@@ -6,6 +6,7 @@ Handles PCI configuration space reading via VFIO and synthetic configuration
 space generation for PCILeech firmware building.
 """
 
+import importlib
 import logging
 import os
 import subprocess
@@ -13,40 +14,14 @@ from dataclasses import dataclass
 from enum import IntEnum
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
+from src.string_utils import (
+    log_debug_safe,
+    log_error_safe,
+    log_info_safe,
+    log_warning_safe,
+)
 
-try:
-    from ..string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-    )
-except ImportError:
-    # Fallback for when string_utils is not available
-    def log_info_safe(logger, template, **kwargs):
-        logger.info(template.format(**kwargs))
-
-    def log_warning_safe(logger, template, **kwargs):
-        logger.warning(template.format(**kwargs))
-
-    def log_error_safe(logger, template, **kwargs):
-        logger.error(template.format(**kwargs))
-
-    def log_debug_safe(logger, template, **kwargs):
-        logger.debug(template.format(**kwargs))
-
-
-# Import device configuration system
-try:
-    from .device_config import DeviceConfiguration, get_device_config
-except ImportError:
-    # Fallback if device config is not available
-    DeviceConfiguration = None
-
-    def get_device_config(
-        profile_name: str = "generic",
-    ) -> Optional[Any]:
-        return None
+from src.device_clone.device_config import DeviceConfiguration, get_device_config
 
 
 logger = logging.getLogger(__name__)
@@ -154,6 +129,21 @@ class BarInfo:
             )
         return self.size_encoding
 
+    @property
+    def size_kb(self) -> float:
+        """Get BAR size in kilobytes."""
+        return self.size / 1024
+
+    @property
+    def size_mb(self) -> float:
+        """Get BAR size in megabytes."""
+        return self.size / (1024 * 1024)
+
+    @property
+    def size_gb(self) -> float:
+        """Get BAR size in gigabytes."""
+        return self.size / (1024 * 1024 * 1024)
+
     def __str__(self) -> str:
         bitness = "64-bit" if self.is_64bit else "32-bit"
         prefetch = "prefetchable" if self.prefetchable else "non-prefetchable"
@@ -224,8 +214,6 @@ class ConfigSpaceManager:
         """Run VFIO diagnostics to help troubleshoot issues."""
         try:
             # Try to import and run VFIO diagnostics if available
-            import importlib
-
             vfio_diag_module = importlib.import_module(
                 "..cli.vfio_diagnostics", package=__name__
             )
@@ -234,14 +222,24 @@ class ConfigSpaceManager:
             )
 
             if run_vfio_diagnostics:
-                log_info_safe(logger, "Running VFIO diagnostics for troubleshooting...")
+                log_info_safe(
+                    logger,
+                    "Running VFIO diagnostics for troubleshooting...",
+                    prefix="VFIO",
+                )
                 run_vfio_diagnostics(self.bdf)
             else:
-                log_warning_safe(logger, "VFIO diagnostics function not found")
+                log_warning_safe(
+                    logger, "VFIO diagnostics function not found", prefix="VFIO"
+                )
         except ImportError:
-            log_warning_safe(logger, "VFIO diagnostics module not available")
+            log_warning_safe(
+                logger, "VFIO diagnostics module not available", prefix="VFIO"
+            )
         except Exception as e:
-            log_warning_safe(logger, "VFIO diagnostics failed: {error}", error=e)
+            log_warning_safe(
+                logger, "VFIO diagnostics failed: {error}", error=e, prefix="VFIO"
+            )
 
     def read_vfio_config_space(self, strict: Optional[bool] = None) -> bytes:
         """
@@ -276,12 +274,13 @@ class ConfigSpaceManager:
     def _read_vfio_strict(self) -> bytes:
         """Read configuration space in strict VFIO mode."""
         try:
-            from ..cli.vfio_handler import VFIOBinder
+            from src.cli.vfio_handler import VFIOBinder
 
             log_info_safe(
                 logger,
                 "Binding device {bdf} to VFIO for configuration space access",
                 bdf=self.bdf,
+                prefix="VFIO",
             )
 
             with VFIOBinder(self.bdf) as vfio_device_path:
@@ -289,10 +288,9 @@ class ConfigSpaceManager:
                     logger,
                     "Successfully bound to VFIO device {vfio_device_path}",
                     vfio_device_path=vfio_device_path,
+                    prefix="VFIO",
                 )
 
-                # TODO: Implement proper VFIO ioctl for config space access
-                # For now, use sysfs method while device is bound to VFIO
                 config_space = self._read_sysfs_config_space()
 
                 log_info_safe(
@@ -750,7 +748,7 @@ class ConfigSpaceManager:
             )
 
     def extract_device_info(self, config_space: bytes) -> Dict[str, Any]:
-        """Extract device information from configuration space with improved structure."""
+        """Extract device information from configuration space with improved structure and resilient lookup."""
         self._validate_config_space_size(config_space)
 
         device_info = self._extract_basic_device_info(config_space)
@@ -1121,42 +1119,64 @@ class ConfigSpaceManager:
         return bar_info
 
     def _log_extracted_device_info(self, device_info: Dict[str, Any]) -> None:
-        """Log extracted device information in a structured way."""
-        vendor_id = device_info["vendor_id"]
-        device_id = device_info["device_id"]
-        class_code = device_info["class_code"]
-        revision_id = device_info["revision_id"]
-        command = device_info["command"]
-        status = device_info["status"]
-        header_type = device_info["header_type"]
-        subsys_vendor_id = device_info["subsystem_vendor_id"]
-        subsys_device_id = device_info["subsystem_device_id"]
-        cache_line_size = device_info["cache_line_size"]
-        latency_timer = device_info["latency_timer"]
-        bist = device_info["bist"]
-        bars = device_info["bars"]
+        """Log extracted device information in a structured way with resilient handling."""
+        # Use .get() with defaults to handle missing fields gracefully
+        vendor_id = device_info.get("vendor_id", 0)
+        device_id = device_info.get("device_id", 0)
+        class_code = device_info.get("class_code", 0)
+        revision_id = device_info.get("revision_id", 0)
+        command = device_info.get("command", 0)
+        status = device_info.get("status", 0)
+        header_type = device_info.get("header_type", 0)
+        subsys_vendor_id = device_info.get("subsystem_vendor_id", 0)
+        subsys_device_id = device_info.get("subsystem_device_id", 0)
+        cache_line_size = device_info.get("cache_line_size", 0)
+        latency_timer = device_info.get("latency_timer", 0)
+        bist = device_info.get("bist", 0)
+        bars = device_info.get("bars", [])
 
-        log_info_safe(logger, "Successfully extracted device information:")
-        log_info_safe(logger, "  Vendor ID: 0x{vendor_id:04x}", vendor_id=vendor_id)
-        log_info_safe(logger, "  Device ID: 0x{device_id:04x}", device_id=device_id)
-        log_info_safe(logger, "  Class Code: 0x{class_code:06x}", class_code=class_code)
         log_info_safe(
-            logger, "  Revision ID: 0x{revision_id:02x}", revision_id=revision_id
+            logger, "Successfully extracted device information:", prefix="INFO"
         )
-        log_info_safe(logger, "  Command: 0x{command:04x}", command=command)
-        log_info_safe(logger, "  Status: 0x{status:04x}", status=status)
         log_info_safe(
-            logger, "  Header Type: 0x{header_type:02x}", header_type=header_type
+            logger, "  Vendor ID: 0x{vendor_id:04x}", vendor_id=vendor_id, prefix="INFO"
+        )
+        log_info_safe(
+            logger, "  Device ID: 0x{device_id:04x}", device_id=device_id, prefix="INFO"
+        )
+        log_info_safe(
+            logger,
+            "  Class Code: 0x{class_code:06x}",
+            class_code=class_code,
+            prefix="INFO",
+        )
+        log_info_safe(
+            logger,
+            "  Revision ID: 0x{revision_id:02x}",
+            revision_id=revision_id,
+            prefix="INFO",
+        )
+        log_info_safe(
+            logger, "  Command: 0x{command:04x}", command=command, prefix="INFO"
+        )
+        log_info_safe(logger, "  Status: 0x{status:04x}", status=status, prefix="INFO")
+        log_info_safe(
+            logger,
+            "  Header Type: 0x{header_type:02x}",
+            header_type=header_type,
+            prefix="INFO",
         )
         log_info_safe(
             logger,
             "  Subsystem Vendor: 0x{subsys_vendor_id:04x}",
             subsys_vendor_id=subsys_vendor_id,
+            prefix="INFO",
         )
         log_info_safe(
             logger,
             "  Subsystem Device: 0x{subsys_device_id:04x}",
             subsys_device_id=subsys_device_id,
+            prefix="INFO",
         )
         log_info_safe(
             logger,
