@@ -15,6 +15,9 @@ from enum import Enum
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Union
 
+from ..utils.validation_constants import (DEVICE_CAPABILITY_ERROR_MESSAGES,
+                                          KNOWN_DEVICE_TYPES)
+
 try:
     import yaml
 
@@ -23,12 +26,8 @@ except ImportError:
     yaml = None
     YAML_AVAILABLE = False
 
-from ..string_utils import (
-    log_debug_safe,
-    log_error_safe,
-    log_info_safe,
-    log_warning_safe,
-)
+from src.string_utils import (log_debug_safe, log_error_safe, log_info_safe,
+                              log_warning_safe, safe_format)
 
 logger = logging.getLogger(__name__)
 
@@ -36,11 +35,39 @@ logger = logging.getLogger(__name__)
 class DeviceType(Enum):
     """PCIe device types with their default configurations."""
 
-    NETWORK = "network"
     AUDIO = "audio"
-    STORAGE = "storage"
     GRAPHICS = "graphics"
-    GENERIC = "generic"
+    MEDIA = "media"
+    NETWORK = "network"
+    PROCESSOR = "processor"
+    STORAGE = "storage"
+    USB = "usb"
+    GENERIC = "generic"  # Keep generic last as it's the fallback
+
+    @classmethod
+    def validate_against_known_types(cls) -> None:
+        """Validate that enum values match centralized constants."""
+        enum_values = {member.value for member in cls}
+        constant_values = set(KNOWN_DEVICE_TYPES)
+
+        if enum_values != constant_values:
+            missing_in_enum = constant_values - enum_values
+            missing_in_constant = enum_values - constant_values
+            error_msg = []
+            if missing_in_enum:
+                error_msg.append(f"Missing in DeviceType enum: {missing_in_enum}")
+            if missing_in_constant:
+                error_msg.append(
+                    f"Missing in KNOWN_DEVICE_TYPES: {missing_in_constant}"
+                )
+            raise ValueError(
+                f"DeviceType enum and KNOWN_DEVICE_TYPES "
+                f"mismatch: {'; '.join(error_msg)}"
+            )
+
+
+# Validate at module load time to catch mismatches early
+DeviceType.validate_against_known_types()
 
 
 class DeviceClass(Enum):
@@ -83,6 +110,40 @@ class DeviceIdentification:
     subsystem_vendor_id: int = 0x0000
     subsystem_device_id: int = 0x0000
 
+    def __post_init__(self):
+        """Convert string values to integers if needed."""
+        # Convert vendor_id
+        if isinstance(self.vendor_id, str):
+            self.vendor_id = self._convert_to_int(self.vendor_id)
+
+        # Convert device_id
+        if isinstance(self.device_id, str):
+            self.device_id = self._convert_to_int(self.device_id)
+
+        # Convert class_code
+        if isinstance(self.class_code, str):
+            self.class_code = self._convert_to_int(self.class_code)
+
+        # Convert subsystem IDs
+        if isinstance(self.subsystem_vendor_id, str):
+            self.subsystem_vendor_id = self._convert_to_int(self.subsystem_vendor_id)
+
+        if isinstance(self.subsystem_device_id, str):
+            self.subsystem_device_id = self._convert_to_int(self.subsystem_device_id)
+
+    @staticmethod
+    def _convert_to_int(value) -> int:
+        """Convert hex string or other value to int."""
+        if isinstance(value, int):
+            return value
+        elif isinstance(value, str):
+            if value.startswith(("0x", "0X")):
+                return int(value, 16)
+            else:
+                return int(value, 0)  # Auto-detect base
+        else:
+            return int(value)
+
     def validate(self) -> None:
         """Validate device identification values."""
         if not (0x0001 <= self.vendor_id <= 0xFFFE):
@@ -171,7 +232,8 @@ class DeviceCapabilities:
     def validate(self) -> None:
         """Validate capability values."""
         # Import here to avoid circular dependency
-        from .payload_size_config import PayloadSizeConfig, PayloadSizeError
+        from src.device_clone.payload_size_config import (PayloadSizeConfig,
+                                                          PayloadSizeError)
 
         # Validate payload size using the new payload size configuration
         try:
@@ -180,35 +242,61 @@ class DeviceCapabilities:
             raise ValueError(str(e))
 
         if not (1 <= self.msi_vectors <= 32):
-            raise ValueError(f"Invalid MSI vector count: {self.msi_vectors}")
+            raise ValueError(
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["invalid_msi_vector_count"],
+                    msi_vectors=self.msi_vectors,
+                )
+            )
 
         if not (0 <= self.msix_vectors <= 2048):
-            raise ValueError(f"Invalid MSI-X vector count: {self.msix_vectors}")
+            raise ValueError(
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["invalid_msix_vector_count"],
+                    msix_vectors=self.msix_vectors,
+                )
+            )
 
         valid_link_widths = [1, 2, 4, 8, 16]
         if self.link_width not in valid_link_widths:
-            raise ValueError(f"Invalid link width: x{self.link_width}")
+            raise ValueError(
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["invalid_link_width"],
+                    link_width=self.link_width,
+                )
+            )
 
         # Validate extended configuration space pointers
         if not (0x100 <= self.ext_cfg_cap_ptr <= 0xFFC):
             raise ValueError(
-                f"Invalid extended config capability pointer: 0x{self.ext_cfg_cap_ptr:03X}"
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["invalid_ext_cfg_cap_ptr"],
+                    ext_cfg_cap_ptr=self.ext_cfg_cap_ptr,
+                )
             )
 
         if not (0x100 <= self.ext_cfg_xp_cap_ptr <= 0xFFC):
             raise ValueError(
-                f"Invalid extended config express capability pointer: 0x{self.ext_cfg_xp_cap_ptr:03X}"
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["invalid_ext_cfg_xp_cap_ptr"],
+                    ext_cfg_xp_cap_ptr=self.ext_cfg_xp_cap_ptr,
+                )
             )
 
         # Ensure pointers are 4-byte aligned
         if self.ext_cfg_cap_ptr % 4 != 0:
             raise ValueError(
-                f"Extended config capability pointer must be 4-byte aligned: 0x{self.ext_cfg_cap_ptr:03X}"
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["ext_cfg_cap_ptr_alignment"],
+                    ext_cfg_cap_ptr=self.ext_cfg_cap_ptr,
+                )
             )
-
         if self.ext_cfg_xp_cap_ptr % 4 != 0:
             raise ValueError(
-                f"Extended config express capability pointer must be 4-byte aligned: 0x{self.ext_cfg_xp_cap_ptr:03X}"
+                safe_format(
+                    DEVICE_CAPABILITY_ERROR_MESSAGES["ext_cfg_xp_cap_ptr_alignment"],
+                    ext_cfg_xp_cap_ptr=self.ext_cfg_xp_cap_ptr,
+                )
             )
 
         # Validate active device configuration
@@ -221,7 +309,7 @@ class DeviceCapabilities:
         Returns:
             cfg_force_mps encoding value (0-5)
         """
-        from .payload_size_config import PayloadSizeConfig
+        from src.device_clone.payload_size_config import PayloadSizeConfig
 
         payload_config = PayloadSizeConfig(self.max_payload_size)
         return payload_config.get_cfg_force_mps()
@@ -233,7 +321,7 @@ class DeviceCapabilities:
         Returns:
             Tuple of (has_issues, warning_message)
         """
-        from .payload_size_config import PayloadSizeConfig
+        from src.device_clone.payload_size_config import PayloadSizeConfig
 
         payload_config = PayloadSizeConfig(self.max_payload_size)
         return payload_config.check_tiny_pcie_algo_issues()
@@ -285,8 +373,12 @@ class DeviceConfiguration:
                 "msix_vectors": self.capabilities.msix_vectors,
                 "supports_msi": self.capabilities.supports_msi,
                 "supports_msix": self.capabilities.supports_msix,
-                "supports_power_management": self.capabilities.supports_power_management,
-                "supports_advanced_error_reporting": self.capabilities.supports_advanced_error_reporting,
+                "supports_power_management": (
+                    self.capabilities.supports_power_management
+                ),
+                "supports_advanced_error_reporting": (
+                    self.capabilities.supports_advanced_error_reporting
+                ),
                 "link_width": self.capabilities.link_width,
                 "link_speed": self.capabilities.link_speed,
                 "ext_cfg_cap_ptr": self.capabilities.ext_cfg_cap_ptr,
@@ -296,29 +388,77 @@ class DeviceConfiguration:
                     "timer_period": self.capabilities.active_device.timer_period,
                     "timer_enable": self.capabilities.active_device.timer_enable,
                     "interrupt_mode": self.capabilities.active_device.interrupt_mode,
-                    "interrupt_vector": self.capabilities.active_device.interrupt_vector,
+                    # NOTE: Avoid trailing commas creating 1-element tuples.
+                    "interrupt_vector": (
+                        self.capabilities.active_device.interrupt_vector
+                    ),
                     "priority": self.capabilities.active_device.priority,
-                    "msi_vector_width": self.capabilities.active_device.msi_vector_width,
+                    "msi_vector_width": (
+                        self.capabilities.active_device.msi_vector_width
+                    ),
                     "msi_64bit_addr": self.capabilities.active_device.msi_64bit_addr,
-                    "num_interrupt_sources": self.capabilities.active_device.num_interrupt_sources,
-                    "default_source_priority": self.capabilities.active_device.default_source_priority,
+                    "num_interrupt_sources": (
+                        self.capabilities.active_device.num_interrupt_sources
+                    ),
+                    "default_source_priority": (
+                        self.capabilities.active_device.default_source_priority
+                    ),
                 },
             },
             "custom_properties": self.custom_properties,
         }
 
 
+def validate_hex_id(value: Any, bit_width: int = 16) -> int:
+    """
+    Validate and convert hex ID string to integer.
+
+    Accepts strings like '0x10ec', '10ec', or decimal strings like '4332'.
+    Auto-detects base: hex if starts with 0x or contains A-F, decimal otherwise.
+    """
+    s = str(value).strip()
+    if s.startswith(("0x", "0X")):
+        s = s[2:]
+        base = 16
+    elif re.match(r"^-?\d+$", s):
+        # Decimal digits with optional leading minus, treat as decimal
+        base = 10
+    elif re.match(r"^-?[0-9A-Fa-f]+$", s):
+        # Contains hex characters with optional leading minus, treat as hex
+        base = 16
+    else:
+        raise ValueError(f"Invalid format: {value}")
+
+    int_value = int(s, base)
+
+    # Check for negative values
+    if int_value < 0:
+        raise ValueError(f"Value {int_value} out of range for {bit_width}-bit field")
+
+    max_value = (1 << bit_width) - 1
+    if not (0 <= int_value <= max_value):
+        raise ValueError(
+            f"Value 0x{int_value:X} out of range for {bit_width}-bit field"
+        )
+
+    return int_value
+
+
 class DeviceConfigManager:
     """Manages device configurations with file loading and validation."""
 
-    # Previously contained hardcoded vendor/device IDs that could create
-    # insecure generic firmware. Use live device detection or explicit
-    # configuration files instead.
     DEFAULT_PROFILES = {}
 
     def __init__(self, config_dir: Optional[Path] = None):
-        """Initialize configuration manager."""
-        self.config_dir = config_dir or Path("configs/devices")
+        """Initialize configuration manager.
+
+        If `config_dir` is provided it enables loading and saving profiles from
+        the filesystem. If `config_dir` is None, no filesystem-backed profiles
+        will be used.
+        """
+        # Do not assume any implicit on-disk directory. Keep as None unless
+        # explicitly provided.
+        self.config_dir = Path(config_dir) if config_dir is not None else None
         self.profiles: Dict[str, DeviceConfiguration] = {}
         self._load_default_profiles()
 
@@ -331,6 +471,41 @@ class DeviceConfigManager:
             count=len(self.DEFAULT_PROFILES),
         )
 
+    # ------------------------------------------------------------------
+    # Internal helpers
+    # ------------------------------------------------------------------
+
+    @staticmethod
+    def _log_preconfigured_config_warning(config_file: Path, file_type: str) -> None:
+        """Emit standardized warning for preconfigured device profiles.
+
+        Avoids duplicated logging blocks for YAML/JSON cases while keeping
+        strong messaging about hardcoded IDs.
+        """
+        # Ensure consistent uppercase file type label.
+        ft_label = file_type.upper()
+        log_warning_safe(logger, "=" * 80)
+        log_warning_safe(
+            logger,
+            "⚠️  WARNING: USING PRECONFIGURED {ft_label} DEVICE CONFIGURATION",
+            ft_label=ft_label,
+        )
+        log_warning_safe(
+            logger,
+            "   Loading device profile from: {config_file}",
+            config_file=str(config_file),
+        )
+        log_warning_safe(
+            logger,
+            "   This uses hardcoded vendor/device IDs that may not be unique!",
+        )
+        log_warning_safe(
+            logger,
+            "   Consider using live device detection instead of {ft_label} configs.",
+            ft_label=ft_label,
+        )
+        log_warning_safe(logger, "=" * 80)
+
     def load_config_file(self, file_path: Union[str, Path]) -> DeviceConfiguration:
         """Load device configuration from file."""
         file_path = Path(file_path)
@@ -341,10 +516,6 @@ class DeviceConfigManager:
         try:
             with open(file_path, "r") as f:
                 if file_path.suffix.lower() in [".yaml", ".yml"]:
-                    if not YAML_AVAILABLE:
-                        raise ImportError(
-                            "PyYAML is required for YAML file support. Install with: pip install PyYAML"
-                        )
                     data = yaml.safe_load(f)  # type: ignore
                 elif file_path.suffix.lower() == ".json":
                     data = json.load(f)
@@ -371,56 +542,103 @@ class DeviceConfigManager:
             raise
 
     def _dict_to_config(self, data: Dict[str, Any]) -> DeviceConfiguration:
-        """Convert dictionary to DeviceConfiguration."""
-        # Require explicit class_code - no default for security
+        """Convert raw dictionary into a validated DeviceConfiguration.
+
+        Handles legacy artifacts (single-element lists for scalar fields)
+        introduced by earlier serialization that wrapped scalars.
+        """
         if "class_code" not in data["identification"]:
             raise ValueError(
                 "class_code must be explicitly specified in device identification"
             )
 
+        def convert_to_int(value: Any) -> int:
+            if isinstance(value, int):
+                return value
+            if isinstance(value, str):
+                s = value.strip()
+                if s.startswith(("0x", "0X")):
+                    return int(s, 16)
+                if re.match(r"^\d+$", s):
+                    return int(s, 10)
+                if re.match(r"^[0-9A-Fa-f]+$", s):
+                    return int(s, 16)
+                try:
+                    return int(s, 0)
+                except ValueError:
+                    return int(s, 16)
+            return int(value)
+
         identification = DeviceIdentification(
-            vendor_id=data["identification"]["vendor_id"],
-            device_id=data["identification"]["device_id"],
-            class_code=data["identification"]["class_code"],
-            subsystem_vendor_id=data["identification"].get(
-                "subsystem_vendor_id", 0x0000
+            vendor_id=convert_to_int(data["identification"]["vendor_id"]),
+            device_id=convert_to_int(data["identification"]["device_id"]),
+            class_code=convert_to_int(data["identification"]["class_code"]),
+            subsystem_vendor_id=convert_to_int(
+                data["identification"].get("subsystem_vendor_id", 0x0000)
             ),
-            subsystem_device_id=data["identification"].get(
-                "subsystem_device_id", 0x0000
+            subsystem_device_id=convert_to_int(
+                data["identification"].get("subsystem_device_id", 0x0000)
             ),
         )
 
         registers = PCIeRegisters(
-            command=data["registers"].get("command", 0x0006),
-            status=data["registers"].get("status", 0x0210),
-            revision_id=data["registers"].get("revision_id", 0x01),
-            cache_line_size=data["registers"].get("cache_line_size", 0x10),
-            latency_timer=data["registers"].get("latency_timer", 0x00),
-            header_type=data["registers"].get("header_type", 0x00),
-            bist=data["registers"].get("bist", 0x00),
+            command=convert_to_int(data["registers"].get("command", 0x0006)),
+            status=convert_to_int(data["registers"].get("status", 0x0210)),
+            revision_id=convert_to_int(data["registers"].get("revision_id", 0x01)),
+            cache_line_size=convert_to_int(
+                data["registers"].get("cache_line_size", 0x10)
+            ),
+            latency_timer=convert_to_int(data["registers"].get("latency_timer", 0x00)),
+            header_type=convert_to_int(data["registers"].get("header_type", 0x00)),
+            bist=convert_to_int(data["registers"].get("bist", 0x00)),
         )
 
-        # Load active device configuration if present
         active_device_data = data["capabilities"].get("active_device", {})
+
+        def _coerce_scalar(value: Any) -> Any:
+            if (
+                isinstance(value, (list, tuple))
+                and len(value) == 1
+                and isinstance(value[0], (int, str))
+            ):
+                return value[0]
+            return value
+
         active_device = ActiveDeviceConfig(
-            enabled=active_device_data.get("enabled", False),
-            timer_period=active_device_data.get("timer_period", 100000),
-            timer_enable=active_device_data.get("timer_enable", True),
-            interrupt_mode=active_device_data.get("interrupt_mode", "msi"),
-            interrupt_vector=active_device_data.get("interrupt_vector", 0),
-            priority=active_device_data.get("priority", 15),
-            msi_vector_width=active_device_data.get("msi_vector_width", 5),
-            msi_64bit_addr=active_device_data.get("msi_64bit_addr", False),
-            num_interrupt_sources=active_device_data.get("num_interrupt_sources", 8),
-            default_source_priority=active_device_data.get(
-                "default_source_priority", 8
+            enabled=_coerce_scalar(active_device_data.get("enabled", False)),
+            timer_period=convert_to_int(
+                _coerce_scalar(active_device_data.get("timer_period", 100000))
+            ),
+            timer_enable=_coerce_scalar(active_device_data.get("timer_enable", True)),
+            interrupt_mode=_coerce_scalar(
+                active_device_data.get("interrupt_mode", "msi")
+            ),
+            interrupt_vector=convert_to_int(
+                _coerce_scalar(active_device_data.get("interrupt_vector", 0))
+            ),
+            priority=convert_to_int(
+                _coerce_scalar(active_device_data.get("priority", 15))
+            ),
+            msi_vector_width=convert_to_int(
+                _coerce_scalar(active_device_data.get("msi_vector_width", 5))
+            ),
+            msi_64bit_addr=_coerce_scalar(
+                active_device_data.get("msi_64bit_addr", False)
+            ),
+            num_interrupt_sources=convert_to_int(
+                _coerce_scalar(active_device_data.get("num_interrupt_sources", 8))
+            ),
+            default_source_priority=convert_to_int(
+                _coerce_scalar(active_device_data.get("default_source_priority", 8))
             ),
         )
 
         capabilities = DeviceCapabilities(
-            max_payload_size=data["capabilities"].get("max_payload_size", 256),
-            msi_vectors=data["capabilities"].get("msi_vectors", 1),
-            msix_vectors=data["capabilities"].get("msix_vectors", 0),
+            max_payload_size=convert_to_int(
+                data["capabilities"].get("max_payload_size", 256)
+            ),
+            msi_vectors=convert_to_int(data["capabilities"].get("msi_vectors", 1)),
+            msix_vectors=convert_to_int(data["capabilities"].get("msix_vectors", 0)),
             supports_msi=data["capabilities"].get("supports_msi", True),
             supports_msix=data["capabilities"].get("supports_msix", False),
             supports_power_management=data["capabilities"].get(
@@ -429,10 +647,14 @@ class DeviceConfigManager:
             supports_advanced_error_reporting=data["capabilities"].get(
                 "supports_advanced_error_reporting", False
             ),
-            link_width=data["capabilities"].get("link_width", 1),
+            link_width=convert_to_int(data["capabilities"].get("link_width", 1)),
             link_speed=data["capabilities"].get("link_speed", "2.5GT/s"),
-            ext_cfg_cap_ptr=data["capabilities"].get("ext_cfg_cap_ptr", 0x100),
-            ext_cfg_xp_cap_ptr=data["capabilities"].get("ext_cfg_xp_cap_ptr", 0x100),
+            ext_cfg_cap_ptr=convert_to_int(
+                data["capabilities"].get("ext_cfg_cap_ptr", 0x100)
+            ),
+            ext_cfg_xp_cap_ptr=convert_to_int(
+                data["capabilities"].get("ext_cfg_xp_cap_ptr", 0x100)
+            ),
             active_device=active_device,
         )
 
@@ -450,57 +672,25 @@ class DeviceConfigManager:
         """Get device profile by name."""
         if name in self.profiles:
             return self.profiles[name]
+        # If a config_dir was provided, attempt to load YAML or JSON files.
+        if self.config_dir is not None:
+            # Try YAML
+            config_file = self.config_dir / f"{name}.yaml"
+            if config_file.exists():
+                self._log_preconfigured_config_warning(config_file, "yaml")
+                config = self.load_config_file(config_file)
+                self.profiles[name] = config
+                return config
 
-        # Try to load from file
-        config_file = self.config_dir / f"{name}.yaml"
-        if config_file.exists():
-            log_warning_safe(logger, "=" * 80)
-            log_warning_safe(
-                logger, "⚠️  WARNING: USING PRECONFIGURED YAML DEVICE CONFIGURATION"
-            )
-            log_warning_safe(
-                logger,
-                "   Loading device profile from: {config_file}",
-                config_file=str(config_file),
-            )
-            log_warning_safe(
-                logger,
-                "   This uses hardcoded vendor/device IDs that may not be unique!",
-            )
-            log_warning_safe(
-                logger,
-                "   Consider using live device detection instead of YAML configs.",
-            )
-            log_warning_safe(logger, "=" * 80)
-            config = self.load_config_file(config_file)
-            self.profiles[name] = config
-            return config
-
-        # Try JSON file
-        config_file = self.config_dir / f"{name}.json"
-        if config_file.exists():
-            log_warning_safe(logger, "=" * 80)
-            log_warning_safe(
-                logger, "⚠️  WARNING: USING PRECONFIGURED JSON DEVICE CONFIGURATION"
-            )
-            log_warning_safe(
-                logger,
-                "   Loading device profile from: {config_file}",
-                config_file=str(config_file),
-            )
-            log_warning_safe(
-                logger,
-                "   This uses hardcoded vendor/device IDs that may not be unique!",
-            )
-            log_warning_safe(
-                logger,
-                "   Consider using live device detection instead of JSON configs.",
-            )
-            log_warning_safe(logger, "=" * 80)
-            config = self.load_config_file(config_file)
-            self.profiles[name] = config
-            return config
+            # Try JSON
+            config_file = self.config_dir / f"{name}.json"
+            if config_file.exists():
+                self._log_preconfigured_config_warning(config_file, "json")
+                config = self.load_config_file(config_file)
+                self.profiles[name] = config
+                return config
 
+        # No profile found either in-memory or on-disk
         raise ValueError(f"Device profile not found: {name}")
 
     def create_profile_from_env(self, name: str) -> DeviceConfiguration:
@@ -533,9 +723,11 @@ class DeviceConfigManager:
                 f"PCIE_{name.upper()}_CLASS_CODE environment variable is required"
             )
 
-        vendor_id = int(vendor_id_env, 0)
-        device_id = int(device_id_env, 0)
-        class_code = int(class_code_env, 0)
+        # Use robust validation/conversion for environment-provided hex IDs
+        vendor_id = validate_hex_id(vendor_id_env, bit_width=16)
+        device_id = validate_hex_id(device_id_env, bit_width=16)
+        # class_code is 24-bit
+        class_code = validate_hex_id(class_code_env, bit_width=24)
 
         identification = DeviceIdentification(
             vendor_id=vendor_id,
@@ -565,7 +757,7 @@ class DeviceConfigManager:
         profiles = list(self.profiles.keys())
 
         # Add profiles from config directory
-        if self.config_dir.exists():
+        if self.config_dir and self.config_dir.exists():
             for file_path in self.config_dir.glob("*.yaml"):
                 profile_name = file_path.stem
                 if profile_name not in profiles:
@@ -583,16 +775,52 @@ class DeviceConfigManager:
     ) -> None:
         """Save device configuration to file."""
         if file_path is None:
+            if self.config_dir is None:
+                raise ValueError(
+                    "No config_dir set: provide file_path or initialize "
+                    "DeviceConfigManager with a config_dir"
+                )
             self.config_dir.mkdir(parents=True, exist_ok=True)
             file_path = self.config_dir / f"{config.name}.yaml"
 
         try:
+            if not YAML_AVAILABLE:
+                raise ImportError(
+                    "PyYAML is required for YAML file support. Install with: "
+                    "pip install PyYAML"
+                )
+
+            def _sanitize(obj: Any):
+                """Recursively sanitize object for safe YAML serialization.
+
+                Converts tuples to lists and strips any non-primitive objects
+                by using their string representation (last resort). This
+                ensures yaml.safe_load can parse without python/tuple tags.
+                """
+                if isinstance(obj, dict):
+                    return {k: _sanitize(v) for k, v in obj.items()}
+                if isinstance(obj, (list, set, tuple)):
+                    return [_sanitize(v) for v in list(obj)]
+                if isinstance(obj, (str, int, float, bool)) or obj is None:
+                    return obj
+                # Fallback: represent unknown types as string
+                try:
+                    return str(obj)
+                except Exception:
+                    return f"<unserializable {type(obj).__name__}>"
+
+            data = _sanitize(config.to_dict())
+
+            # Use safe_dump so that safe_load works symmetrically
+            assert yaml is not None  # for type checker
             with open(file_path, "w") as f:
-                if not YAML_AVAILABLE:
-                    raise ImportError(
-                        "PyYAML is required for YAML file support. Install with: pip install PyYAML"
-                    )
-                yaml.dump(config.to_dict(), f, default_flow_style=False, indent=2)  # type: ignore
+                yaml.safe_dump(
+                    data,
+                    f,
+                    default_flow_style=False,
+                    indent=2,
+                    sort_keys=True,
+                )
 
             # Add the profile to the in-memory profiles dictionary
             self.profiles[config.name] = config
@@ -652,32 +880,6 @@ def get_device_config(profile_name: str) -> Optional[DeviceConfiguration]:
         return None
 
 
-def validate_hex_id(value: str, bit_width: int = 16) -> int:
-    """Validate and convert hex ID string to integer."""
-    if isinstance(value, int):
-        return value
-
-    # Remove 0x prefix if present
-    if value.startswith(("0x", "0X")):
-        value = value[2:]
-
-    # Validate hex format
-    if not re.match(r"^[0-9A-Fa-f]+$", value):
-        raise ValueError(f"Invalid hex format: {value}")
-
-    # Convert to integer
-    int_value = int(value, 16)
-
-    # Validate range
-    max_value = (1 << bit_width) - 1
-    if not (0 <= int_value <= max_value):
-        raise ValueError(
-            f"Value 0x{int_value:X} out of range for {bit_width}-bit field"
-        )
-
-    return int_value
-
-
 def generate_device_state_machine(registers: List[Dict[str, Any]]) -> Dict[str, Any]:
     """Generate device-level state machine."""
     try:
@@ -689,7 +891,12 @@ def generate_device_state_machine(registers: List[Dict[str, Any]]) -> Dict[str,
             "device_states": ["INIT", "READY", "ACTIVE", "ERROR"],
             "register_count": len(registers),
             "state_transitions": [
-                {"from": "INIT", "to": "READY", "trigger": "initialization_complete"},
+                {
+                    "from": "INIT",
+                    "to": "READY",
+                    "trigger": "initialization_complete",
+                },
+                # Short descriptions below remain compact; line wrapped above.
                 {"from": "READY", "to": "ACTIVE", "trigger": "operation_start"},
                 {"from": "ACTIVE", "to": "READY", "trigger": "operation_complete"},
                 {"from": "*", "to": "ERROR", "trigger": "error_condition"},
