@@ -10,13 +10,19 @@ import logging
 from pathlib import Path
 from typing import Any, Dict, List, Optional
 
-from ..string_utils import log_info_safe
-from .behavior_profiler import BehaviorProfiler
-from .manufacturing_variance import (
+from src.device_clone.behavior_profiler import BehaviorProfiler
+from src.device_clone.manufacturing_variance import (
     DeviceClass,
     ManufacturingVarianceSimulator,
     VarianceModel,
 )
+from src.string_utils import (
+    log_debug_safe,
+    log_error_safe,
+    log_info_safe,
+    log_warning_safe,
+    safe_format,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -30,12 +36,14 @@ class VarianceManager:
         self.variance_simulator = None
         self.behavior_profiler = None
 
-        # Use provided fallback manager or create a default one
+        # Use provided fallback manager or the shared/global one
         if fallback_manager is None:
             try:
-                from .fallback_manager import FallbackManager
+                from src.device_clone.fallback_manager import (
+                    get_global_fallback_manager,
+                )
 
-                self.fallback_manager = FallbackManager(mode="none")
+                self.fallback_manager = get_global_fallback_manager(mode="none")
             except ImportError:
                 self.fallback_manager = None
         else:
@@ -48,7 +56,7 @@ class VarianceManager:
         try:
             if not DeviceClass or not VarianceModel:
                 error_msg = "Manufacturing variance modules not available"
-                logger.warning(error_msg)
+                log_warning_safe(logger, safe_format("{msg}", msg=error_msg))
 
                 # Check with fallback manager if available
                 if (
@@ -59,7 +67,10 @@ class VarianceManager:
                         "Variance simulation enhances realism but isn't critical for functionality.",
                     )
                 ):
-                    logger.error("Manufacturing variance fallback denied by policy")
+                    log_error_safe(
+                        logger,
+                        safe_format("Manufacturing variance fallback denied by policy"),
+                    )
 
                 return variance_files
 
@@ -108,13 +119,18 @@ class VarianceManager:
 
             log_info_safe(
                 logger,
-                "Applied manufacturing variance for {device_class}",
-                device_class=device_class.value,
+                safe_format(
+                    "Applied manufacturing variance for {device_class}",
+                    device_class=device_class.value,
+                ),
+                prefix="VAR",
             )
 
         except Exception as e:
-            error_msg = f"Error applying manufacturing variance: {e}"
-            logger.error(error_msg)
+            error_msg = safe_format(
+                "Error applying manufacturing variance: {error}", error=e
+            )
+            log_error_safe(logger, error_msg, prefix="VAR")
 
             # Check with fallback manager if available
             if self.fallback_manager:
@@ -129,10 +145,26 @@ class VarianceManager:
     def run_behavior_profiling(
         self, device_info: Dict[str, Any], duration: int = 30
     ) -> Optional[str]:
-        """Run behavior profiling if available."""
+        """
+        Runs behavior profiling on the specified device for a given duration and saves the results to a JSON file.
+
+        Args:
+            device_info (Dict[str, Any]): Information about the device to profile.
+            duration (int, optional): Duration in seconds to run the profiling. Defaults to 30.
+
+        Returns:
+            Optional[str]: Path to the saved behavior profile JSON file if profiling succeeds, otherwise None.
+
+        Behavior:
+            - Attempts to run behavior profiling using the BehaviorProfiler if available.
+            - If BehaviorProfiler is not available or an error occurs, logs the issue and consults the fallback manager if configured.
+            - Captures and serializes device behavior data including register accesses, timing patterns, state transitions, power states, and interrupt patterns.
+            - Saves the serialized profile data to 'behavior_profile.json' in the output directory.
+            - Returns the path to the saved profile file or None if profiling was not performed.
+        """
         if not BehaviorProfiler:
             error_msg = "Behavior profiler not available"
-            logger.warning(error_msg)
+            log_warning_safe(logger, safe_format("{msg}", msg=error_msg))
 
             # Check with fallback manager if available
             if self.fallback_manager and not self.fallback_manager.confirm_fallback(
@@ -140,12 +172,23 @@ class VarianceManager:
                 error_msg,
                 "Behavior profiling enhances device emulation but isn't critical for functionality.",
             ):
-                logger.error("Behavior profiling fallback denied by policy")
+                log_error_safe(
+                    logger,
+                    safe_format("Behavior profiling fallback denied by policy"),
+                    prefix="PROF",
+                )
 
             return None
 
         try:
-            logger.info(f"Starting behavior profiling for {duration} seconds")
+            log_info_safe(
+                logger,
+                safe_format(
+                    "Starting behavior profiling for {duration} seconds",
+                    duration=duration,
+                ),
+                prefix="PROF",
+            )
             self.behavior_profiler = BehaviorProfiler(self.bdf)
 
             # Capture behavior profile
@@ -188,12 +231,18 @@ class VarianceManager:
             with open(profile_file, "w") as f:
                 json.dump(profile_dict, f, indent=2)
 
-            logger.info(f"Behavior profiling completed, saved to {profile_file}")
+            log_info_safe(
+                logger,
+                safe_format(
+                    "Behavior profiling completed, saved to {file}", file=profile_file
+                ),
+                prefix="PROF",
+            )
             return str(profile_file)
 
         except Exception as e:
             error_msg = f"Error during behavior profiling: {e}"
-            logger.error(error_msg)
+            log_error_safe(logger, safe_format("{msg}", msg=error_msg))
 
             # Check with fallback manager if available
             if self.fallback_manager and not self.fallback_manager.confirm_fallback(
@@ -201,7 +250,11 @@ class VarianceManager:
                 str(e),
                 "Without behavior profiling, the generated firmware may not accurately reflect device timing patterns.",
             ):
-                logger.error("Behavior profiling fallback denied by policy")
+                log_error_safe(
+                    logger,
+                    safe_format("Behavior profiling fallback denied by policy"),
+                    prefix="PROF",
+                )
 
             return None
 
