@@ -4,7 +4,6 @@
 This utility clones, updates, and queries board‑specific files from the
 `pcileech-fpga` repository.  It is written to be imported by other tools but can
 also be executed directly to verify that the repository is present on the local
-machine.
 It provides methods to ensure the repository is cloned, check for updates, and
 retrieve board paths and XDC files for various PCILeech boards.
 """
@@ -16,16 +15,11 @@ import shutil as _shutil
 import subprocess as _sp
 import time as _time
 from pathlib import Path
-from typing import List
+from typing import List, Optional
 
-# Import project logging and string utilities
 from ..log_config import get_logger
-from ..string_utils import (
-    log_debug_safe,
-    log_error_safe,
-    log_info_safe,
-    log_warning_safe,
-)
+from ..string_utils import (log_debug_safe, log_error_safe, log_info_safe,
+                            log_warning_safe, utc_timestamp)
 
 ###############################################################################
 # Configuration constants - override with environment vars if desired.
@@ -55,11 +49,17 @@ _logger = get_logger(__name__)
 
 
 def _run(
-    cmd: List[str], *, cwd: Path | None = None, env: dict | None = None
+    cmd: List[str], *, cwd: Optional[Path] = None, env: Optional[dict] = None
 ) -> _sp.CompletedProcess:
     """Run *cmd* and return the completed process, raising on error."""
     log_debug_safe(_logger, "Running {cmd} (cwd={cwd})", cmd=cmd, cwd=cwd)
-    return _sp.run(cmd, cwd=str(cwd) if cwd else None, env=env, check=True, text=True)
+    return _sp.run(
+        cmd,
+        cwd=str(cwd) if cwd else None,
+        env=env,
+        check=True,
+        text=True,
+    )
 
 
 def _git_available() -> bool:
@@ -111,7 +111,9 @@ class RepoManager:
         return repo_path
 
     @classmethod
-    def get_board_path(cls, board_type: str, *, repo_root: Path | None = None) -> Path:
+    def get_board_path(
+        cls, board_type: str, *, repo_root: Optional[Path] = None
+    ) -> Path:
         repo_root = repo_root or cls.ensure_repo()
         mapping = {
             "35t": repo_root / "PCIeSquirrel",
@@ -132,17 +134,24 @@ class RepoManager:
             path = mapping[board_type]
         except KeyError as exc:
             raise RuntimeError(
-                f"Unknown board type '{board_type}'.  Known types: {', '.join(mapping)}"
+                (
+                    "Unknown board type '{bt}'.  Known types: {known}".format(
+                        bt=board_type, known=", ".join(mapping)
+                    )
+                )
             ) from exc
         if not path.exists():
             raise RuntimeError(
-                f"Board directory {path} does not exist.  Repository may be incomplete."
+                (
+                    "Board directory {p} does not exist.  Repository may be "
+                    "incomplete."
+                ).format(p=path)
             )
         return path
 
     @classmethod
     def get_xdc_files(
-        cls, board_type: str, *, repo_root: Path | None = None
+        cls, board_type: str, *, repo_root: Optional[Path] = None
     ) -> List[Path]:
         board_dir = cls.get_board_path(board_type, repo_root=repo_root)
         search_roots = [
@@ -170,7 +179,7 @@ class RepoManager:
 
     @classmethod
     def read_combined_xdc(
-        cls, board_type: str, *, repo_root: Path | None = None
+        cls, board_type: str, *, repo_root: Optional[Path] = None
     ) -> str:
         files = cls.get_xdc_files(board_type, repo_root=repo_root)
         parts = [
@@ -218,7 +227,12 @@ class RepoManager:
         if stamp.exists():
             try:
                 last = _dt.datetime.fromisoformat(stamp.read_text().strip())
-                need_update = (_dt.datetime.now() - last).days >= UPDATE_INTERVAL_DAYS
+                # Ensure both datetimes are timezone-aware for comparison
+                now = _dt.datetime.now(_dt.timezone.utc)
+                if last.tzinfo is None:
+                    # If parsed datetime is naive, assume it's UTC
+                    last = last.replace(tzinfo=_dt.timezone.utc)
+                need_update = (now - last).days >= UPDATE_INTERVAL_DAYS
             except ValueError:
                 pass  # treat malformed stamp as out‑of‑date
         if not need_update:
@@ -238,7 +252,7 @@ class RepoManager:
 
         try:
             _run(["git", "-C", str(path), "pull", "--rebase", "--autostash"])
-            stamp.write_text(_dt.datetime.now().isoformat())
+            stamp.write_text(utc_timestamp())
         except Exception as exc:
             log_warning_safe(_logger, "Git pull failed: {error}", error=exc)
 
@@ -265,11 +279,14 @@ class RepoManager:
                     ["git", "clone", "--depth", "1", repo_url, str(dst)],
                     cwd=dst.parent,
                 )
-                (dst / ".last_update").write_text(_dt.datetime.now().isoformat())
+                (dst / ".last_update").write_text(utc_timestamp())
                 return
             except Exception as exc:
-                if dst.exists():
+                # Remove failed clone directory if it exists
+                try:
                     _shutil.rmtree(dst, ignore_errors=True)
+                except Exception:
+                    pass  # Ignore errors during cleanup
                 log_warning_safe(
                     _logger,
                     "Clone attempt {attempts} failed: {error}",
@@ -294,7 +311,7 @@ def get_repo_manager() -> type[RepoManager]:
     return RepoManager
 
 
-def get_xdc_files(board_type: str, *, repo_root: Path | None = None) -> List[Path]:
+def get_xdc_files(board_type: str, *, repo_root: Optional[Path] = None) -> List[Path]:
     """Wrapper function to get XDC files for a board type.
 
     Args:
@@ -307,7 +324,7 @@ def get_xdc_files(board_type: str, *, repo_root: Path | None = None) -> List[Pat
     return RepoManager.get_xdc_files(board_type, repo_root=repo_root)
 
 
-def read_combined_xdc(board_type: str, *, repo_root: Path | None = None) -> str:
+def read_combined_xdc(board_type: str, *, repo_root: Optional[Path] = None) -> str:
     """Wrapper function to read combined XDC content for a board type.
 
     Args:
@@ -321,9 +338,9 @@ def read_combined_xdc(board_type: str, *, repo_root: Path | None = None) -> str:
 
 
 def is_repository_accessible(
-    board_type: str | None = None, *, repo_root: Path | None = None
+    board_type: Optional[str] = None, *, repo_root: Optional[Path] = None
 ) -> bool:
-    """Check if the repository is accessible and optionally if a specific board exists.
+    """Check repo accessibility; optionally verify a specific board exists.
 
     Args:
         board_type: Optional board type to check for specific board accessibility
