@@ -9,50 +9,18 @@ extended capabilities.
 """
 
 import logging
-from typing import Dict, Iterator, List, Optional, Set
+from typing import Dict, Iterator, Optional, Set
 
-try:
-    from ..string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-        safe_format,
-    )
-except ImportError:
-    # Fallback for script execution
-    import sys
-    from pathlib import Path
-
-    src_dir = Path(__file__).parent.parent
-    if str(src_dir) not in sys.path:
-        sys.path.insert(0, str(src_dir))
-    from string_utils import (
-        log_debug_safe,
-        log_error_safe,
-        log_info_safe,
-        log_warning_safe,
-        safe_format,
-    )
-
-from .constants import (
-    EXTENDED_CAPABILITY_NAMES,
-    PCI_CAPABILITIES_POINTER,
-    PCI_CONFIG_SPACE_MIN_HEX_CHARS,
-    PCI_CONFIG_SPACE_MIN_SIZE,
-    PCI_EXT_CAP_ALIGNMENT,
-    PCI_EXT_CAP_ID_MASK,
-    PCI_EXT_CAP_NEXT_PTR_MASK,
-    PCI_EXT_CAP_NEXT_PTR_SHIFT,
-    PCI_EXT_CAP_START,
-    PCI_EXT_CAP_VERSION_MASK,
-    PCI_EXT_CAP_VERSION_SHIFT,
-    PCI_EXT_CONFIG_SPACE_END,
-    PCI_STATUS_CAP_LIST,
-    PCI_STATUS_REGISTER,
-    STANDARD_CAPABILITY_NAMES,
-    TWO_BYTE_HEADER_CAPABILITIES,
-)
+from ..string_utils import log_info_safe, log_warning_safe, safe_format
+from .constants import (EXTENDED_CAPABILITY_NAMES, PCI_CAPABILITIES_POINTER,
+                        PCI_CONFIG_SPACE_MIN_HEX_CHARS,
+                        PCI_CONFIG_SPACE_MIN_SIZE, PCI_EXT_CAP_ALIGNMENT,
+                        PCI_EXT_CAP_ID_MASK, PCI_EXT_CAP_NEXT_PTR_MASK,
+                        PCI_EXT_CAP_NEXT_PTR_SHIFT, PCI_EXT_CAP_START,
+                        PCI_EXT_CAP_VERSION_MASK, PCI_EXT_CAP_VERSION_SHIFT,
+                        PCI_EXT_CONFIG_SPACE_END, PCI_STATUS_CAP_LIST,
+                        PCI_STATUS_REGISTER, STANDARD_CAPABILITY_NAMES,
+                        TWO_BYTE_HEADER_CAPABILITIES)
 from .types import CapabilityInfo, CapabilityType
 
 logger = logging.getLogger(__name__)
