@@ -8,8 +8,29 @@ cause syntax errors when split across lines.
 """
 
 import logging
-from datetime import datetime
-from typing import Any, Dict, List, Optional
+
+import os
+
+from datetime import datetime, timezone
+
+from typing import Any, Dict, List, Optional, Tuple
+
+# Define the project name constant locally to avoid circular imports
+VIVADO_PROJECT_NAME = "pcileech_firmware"
+
+
+def get_project_name() -> str:
+    """
+    Return the canonical project name for Vivado/PCILeech builds.
+    Always use this function or VIVADO_PROJECT_NAME for project naming.
+    """
+    return VIVADO_PROJECT_NAME
+
+
+# Short constants to keep lines within linter limits while reusing the same banner
+# Note: dynamic borders are constructed in helpers to satisfy line-length rules.
+SV_HEADER_BAR = "//=="
+TCL_HEADER_BAR = "#=="
 
 
 def safe_format(template: str, prefix: Optional[str] = None, **kwargs: Any) -> str:
@@ -108,7 +129,20 @@ def safe_log_format(
         level_str = level_map.get(log_level, "UNKNOWN")
 
         padded_message = format_padded_message(formatted_message, level_str)
-        logger.log(log_level, padded_message)
+        # Call the specific logger method so mocks like mock_logger.info are
+        # invoked during tests (instead of logger.log which doesn't call them).
+        if log_level == logging.INFO:
+            logger.info(padded_message)
+        elif log_level == logging.WARNING:
+            logger.warning(padded_message)
+        elif log_level == logging.DEBUG:
+            logger.debug(padded_message)
+        elif log_level == logging.ERROR:
+            logger.error(padded_message)
+        elif log_level == logging.CRITICAL:
+            logger.critical(padded_message)
+        else:
+            logger.log(log_level, padded_message)
     except Exception as e:
         # Fallback to basic logging if formatting fails
         error_msg = f"Failed to format log message: {e}"
@@ -119,7 +153,8 @@ def safe_log_format(
         if prefix:
             fallback_message = f"[{prefix}] {fallback_message}"
         padded_fallback = format_padded_message(fallback_message, "ERROR")
-        logger.log(log_level, padded_fallback)
+        # Use error to ensure visibility of the fallback
+        logger.error(padded_fallback)
 
 
 def safe_print_format(template: str, prefix: str, **kwargs: Any) -> None:
@@ -278,6 +313,43 @@ def get_short_timestamp() -> str:
     return datetime.now().strftime("%H:%M:%S")
 
 
+def utc_timestamp(
+    precise: bool = False,
+    env_var: str = "BUILD_TIMESTAMP",
+    fallback: str = "2024-01-01T00:00:00Z",
+) -> str:
+    """Return a standardized UTC ISO-8601 timestamp with trailing Z.
+
+    Args:
+        precise: Include microseconds if True (default False)
+        env_var: Environment variable that, if set, overrides the timestamp
+        fallback: Fallback timestamp if generation fails
+
+    Behavior:
+        - If the environment variable exists, it's validated (appends 'Z' if
+          missing and contains no timezone). Returned as-is otherwise.
+        - Uses timezone-aware UTC datetime; strips microseconds unless
+          precise=True.
+        - Always normalizes '+00:00' suffix to 'Z'.
+    """
+    try:
+        override = os.getenv(env_var)
+        if override:
+            # Basic normalization: ensure trailing Z if no timezone specified
+            if override.endswith("Z") or override.endswith("z"):
+                return override.rstrip("zZ") + "Z"
+            if "+" in override or override.endswith("Z"):
+                return override.replace("+00:00", "Z")
+            return override + "Z"
+
+        dt = datetime.now(timezone.utc)
+        if not precise:
+            dt = dt.replace(microsecond=0)
+        return dt.isoformat().replace("+00:00", "Z")
+    except Exception:
+        return fallback
+
+
 def format_padded_message(message: str, log_level: str) -> str:
     """
     Format a message with padding based on log level.
@@ -310,40 +382,48 @@ def format_padded_message(message: str, log_level: str) -> str:
 
 
 # Convenience functions for common logging patterns
+
+
 def log_info_safe(
-    logger: logging.Logger, template: str, prefix: Optional[str] = None, **kwargs: Any
+    logger: logging.Logger,
+    template: str,
+    prefix: Optional[str] = None,
+    **kwargs: Any,
 ) -> None:
-    """Convenience function for safe INFO level logging with padding."""
-    formatted_message = safe_format(template, prefix=prefix, **kwargs)
-    padded_message = format_padded_message(formatted_message, "INFO")
-    logger.info(padded_message)
+    """Convenience function for safe INFO level logging."""
+    # Use the centralized safe_log_format to ensure padding/timestamp and
+    # consistent behavior across log levels.
+    safe_log_format(logger, logging.INFO, template, prefix=prefix, **kwargs)
 
 
 def log_error_safe(
-    logger: logging.Logger, template: str, prefix: Optional[str] = None, **kwargs: Any
+    logger: logging.Logger,
+    template: str,
+    prefix: Optional[str] = None,
+    **kwargs: Any,
 ) -> None:
-    """Convenience function for safe ERROR level logging with padding."""
-    formatted_message = safe_format(template, prefix=prefix, **kwargs)
-    padded_message = format_padded_message(formatted_message, "ERROR")
-    logger.error(padded_message)
+    """Convenience function for safe ERROR level logging."""
+    safe_log_format(logger, logging.ERROR, template, prefix=prefix, **kwargs)
 
 
 def log_warning_safe(
-    logger: logging.Logger, template: str, prefix: Optional[str] = None, **kwargs: Any
+    logger: logging.Logger,
+    template: str,
+    prefix: Optional[str] = None,
+    **kwargs: Any,
 ) -> None:
-    """Convenience function for safe WARNING level logging with padding."""
-    formatted_message = safe_format(template, prefix=prefix, **kwargs)
-    padded_message = format_padded_message(formatted_message, "WARNING")
-    logger.warning(padded_message)
+    """Convenience function for safe WARNING level logging."""
+    safe_log_format(logger, logging.WARNING, template, prefix=prefix, **kwargs)
 
 
 def log_debug_safe(
-    logger: logging.Logger, template: str, prefix: Optional[str] = None, **kwargs: Any
+    logger: logging.Logger,
+    template: str,
+    prefix: Optional[str] = None,
+    **kwargs: Any,
 ) -> None:
-    """Convenience function for safe DEBUG level logging with padding."""
-    formatted_message = safe_format(template, prefix=prefix, **kwargs)
-    padded_message = format_padded_message(formatted_message, "DEBUG")
-    logger.debug(padded_message)
+    """Convenience function for safe DEBUG level logging."""
+    safe_log_format(logger, logging.DEBUG, template, prefix=prefix, **kwargs)
 
 
 def generate_sv_header_comment(
@@ -374,14 +454,21 @@ def generate_sv_header_comment(
         ...     "Device Configuration Module",
         ...     vendor_id="1234", device_id="5678", board="AC701"
         ... )
-        '//==============================================================================\\n// Device Configuration Module - Generated for 1234:5678\\n// Board: AC701\\n//=============================================================================='
+        '//... Device Configuration Module - Generated for 1234:5678 ...'
 
         >>> generate_sv_header_comment("PCIe Controller Module")
-        '//==============================================================================\\n// PCIe Controller Module\\n//=============================================================================='
+        '//... PCIe Controller Module ...'
     """
-    lines = [
-        "//=============================================================================="
-    ]
+    from src.utils.validation_constants import SV_FILE_HEADER
+
+    # Use the first line of the standardized header as a base (single-line banner)
+    header_base = (
+        SV_FILE_HEADER.split("\n")[0] if "\n" in SV_FILE_HEADER else SV_FILE_HEADER
+    )
+
+    # Build border line dynamically to satisfy line-length rules
+    sv_border = "//" + "=" * 78
+    lines = [sv_border, header_base]
 
     # Build the main title line
     if vendor_id and device_id:
@@ -401,9 +488,7 @@ def generate_sv_header_comment(
             display_key = key.replace("_", " ").title()
             lines.append(f"// {display_key}: {value}")
 
-    lines.append(
-        "//=============================================================================="
-    )
+    lines.append(sv_border)
 
     return "\n".join(lines)
 
@@ -441,11 +526,18 @@ def generate_tcl_header_comment(
         ...     vendor_id="1234", device_id="5678",
         ...     class_code="0200", board="AC701"
         ... )
-        '#==============================================================================\\n# PCILeech Firmware Build Script\\n# Generated for device 1234:5678 (Class: 0200)\\n# Board: AC701\\n#=============================================================================='
+    '# ... PCILeech Firmware Build Script ...'
     """
-    lines = [
-        "#=============================================================================="
-    ]
+    from src.utils.validation_constants import TCL_FILE_HEADER
+
+    # Use the standardized header as a base (single-line banner)
+    header_base = (
+        TCL_FILE_HEADER.split("\n")[0] if "\n" in TCL_FILE_HEADER else TCL_FILE_HEADER
+    )
+
+    # Build border line dynamically to satisfy line-length rules
+    tcl_border = "#" + "=" * 78
+    lines = [tcl_border, header_base]
 
     # Build the main title line
     lines.append(f"# {title}")
@@ -472,10 +564,79 @@ def generate_tcl_header_comment(
             display_key = key.replace("_", " ").title()
             lines.append(f"# {display_key}: {value}")
 
-    lines.append(
-        "#=============================================================================="
+    lines.append(tcl_border)
+
+    return "\n".join(lines)
+
+
+def generate_hex_header_comment(
+    title: str,
+    total_bytes: Optional[int] = None,
+    total_dwords: Optional[int] = None,
+    vendor_id: Optional[str] = None,
+    device_id: Optional[str] = None,
+    class_code: Optional[str] = None,
+    board: Optional[str] = None,
+    **kwargs: Any,
+) -> str:
+    """
+    Generate a standardized HEX file header comment block.
+
+    This creates a consistent header used for .hex files intended for
+    $readmemh initialization, aligning with other header styles.
+
+    Args:
+        title: Main title/description (e.g., "config_space_init.hex - ...")
+        total_bytes: Optional total byte count included when provided
+        total_dwords: Optional total dword count included when provided
+        vendor_id: Optional vendor ID to display
+        device_id: Optional device ID to display
+        class_code: Optional class code to display
+        board: Optional board identifier
+        **kwargs: Additional key-value pairs to include as comments
+
+    Returns:
+        Formatted HEX header comment block
+    """
+    from src.utils.validation_constants import HEX_FILE_HEADER
+
+    # First line from standardized header as base
+    header_base = (
+        HEX_FILE_HEADER.split("\n")[0] if "\n" in HEX_FILE_HEADER else HEX_FILE_HEADER
     )
 
+    # Dynamic border line to respect line-length rules
+    hex_border = "//" + "=" * 78
+    lines = [hex_border, header_base]
+
+    # Title
+    lines.append(f"// {title}")
+
+    # Optional device information
+    if vendor_id and device_id:
+        dev_line = f"// Generated for device {vendor_id}:{device_id}"
+        if class_code:
+            dev_line += f" (Class: {class_code})"
+        lines.append(dev_line)
+
+    # Optional board info
+    if board:
+        lines.append(f"// Board: {board}")
+
+    # Totals if provided
+    if total_bytes is not None and total_dwords is not None:
+        lines.append(f"// Total size: {total_bytes} bytes ({total_dwords} dwords)")
+    elif total_bytes is not None:
+        lines.append(f"// Total size: {total_bytes} bytes")
+
+    # Additional key/value pairs
+    for key, value in kwargs.items():
+        if value is not None:
+            display_key = key.replace("_", " ").title()
+            lines.append(f"// {display_key}: {value}")
+
+    lines.append(hex_border)
+
     return "\n".join(lines)
 
 
@@ -755,3 +916,49 @@ def format_raw_bar_table(bars: List[Any], device_bdf: str) -> str:
     lines.append(bottom_border)
 
     return "\n".join(lines)
+
+
+def format_kv_table(rows: List[Tuple[str, str]], title: str) -> str:
+    """
+    Format a simple key/value table with a banner title.
+
+    Args:
+        rows: List of (key, value) pairs to display
+        title: Title to show in the banner
+
+    Returns:
+        Box-drawn table string
+    """
+    if rows is None:
+        rows = []
+
+    headers = ["Field", "Value"]
+    col_widths = [len(headers[0]), len(headers[1])]
+    for k, v in rows:
+        col_widths[0] = max(col_widths[0], len(str(k)))
+        col_widths[1] = max(col_widths[1], len(str(v)))
+
+    top = "┌" + "┬".join("─" * (w + 2) for w in col_widths) + "┐"
+    hdr = (
+        "│ "
+        + f"{headers[0]:<{col_widths[0]}}"
+        + " │ "
+        + f"{headers[1]:<{col_widths[1]}}"
+        + " │"
+    )
+    sep = "├" + "┼".join("─" * (w + 2) for w in col_widths) + "┤"
+    lines = [top, hdr, sep]
+    for k, v in rows:
+        k_str = str(k)
+        v_str = str(v)
+        lines.append(
+            "│ "
+            + f"{k_str:<{col_widths[0]}}"
+            + " │ "
+            + f"{v_str:<{col_widths[1]}}"
+            + " │"
+        )
+    bottom = "└" + "┴".join("─" * (w + 2) for w in col_widths) + "┘"
+    title_bar = f"── {title} "
+    banner = "┌" + title_bar + "─" * max(0, sum(col_widths) + 5 - len(title_bar)) + "┐"
+    return "\n".join([banner] + lines + [bottom])
