@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 """
 Vivado Application Handling Module
 
@@ -8,22 +9,15 @@ This module contains utilities and tools for working with Xilinx Vivado:
 - pcileech_build_integration: Integration with pcileech-fpga repository
 """
 
-from .pcileech_build_integration import (
-    PCILeechBuildIntegration,
-    integrate_pcileech_build,
-)
-from .vivado_error_reporter import (
-    VivadoErrorReporter,
-    create_enhanced_vivado_runner,
-    run_vivado_with_error_reporting,
-)
-from .vivado_utils import (
-    debug_vivado_search,
-    find_vivado_installation,
-    get_vivado_executable,
-    get_vivado_version,
-    run_vivado_command,
-)
+from .pcileech_build_integration import (PCILeechBuildIntegration,
+                                         integrate_pcileech_build)
+from .vivado_error_reporter import (VivadoErrorReporter,
+                                    create_enhanced_vivado_runner,
+                                    run_vivado_with_error_reporting)
+from .vivado_runner import VivadoRunner, create_vivado_runner
+from .vivado_utils import (debug_vivado_search, find_vivado_installation,
+                           get_vivado_executable, get_vivado_version,
+                           run_vivado_command)
 
 __all__ = [
     # vivado_utils exports
@@ -36,6 +30,9 @@ __all__ = [
     "VivadoErrorReporter",
     "run_vivado_with_error_reporting",
     "create_enhanced_vivado_runner",
+    # vivado_runner exports (new simplified interface)
+    "VivadoRunner",
+    "create_vivado_runner",
     # pcileech_build_integration exports
     "PCILeechBuildIntegration",
     "integrate_pcileech_build",
