@@ -79,6 +79,7 @@ class PCILeechBuildIntegration:
             logger,
             "Preparing build environment for {board_name}",
             board_name=board_name,
+            prefix="BUILD",
         )
 
         # Create board-specific output directory
@@ -126,9 +127,12 @@ class PCILeechBuildIntegration:
                     logger,
                     "Copied XDC file: {xdc_file_name}",
                     xdc_file_name=xdc_file.name,
+                    prefix="BUILD",
                 )
         except Exception as e:
-            log_warning_safe(logger, "Failed to copy XDC files: {error}", error=e)
+            log_warning_safe(
+                logger, "Failed to copy XDC files: {error}", error=e, prefix="BUILD"
+            )
 
         return copied_files
 
@@ -159,6 +163,7 @@ class PCILeechBuildIntegration:
                     "Failed to copy source file {src_file}: {error}",
                     src_file=str(src_file),
                     error=e,
+                    prefix="BUILD",
                 )
 
         # Also copy core PCILeech files
@@ -168,13 +173,19 @@ class PCILeechBuildIntegration:
             try:
                 shutil.copy2(filepath, dest_path)
                 copied_files.append(dest_path)
-                log_info_safe(logger, "Copied core file: {filename}", filename=filename)
+                log_info_safe(
+                    logger,
+                    "Copied core file: {filename}",
+                    filename=filename,
+                    prefix="BUILD",
+                )
             except Exception as e:
                 log_warning_safe(
                     logger,
                     "Failed to copy core file {filename}: {error}",
                     filename=filename,
                     error=e,
+                    prefix="BUILD",
                 )
 
         return copied_files
@@ -210,10 +221,13 @@ class PCILeechBuildIntegration:
                 logger,
                 "Using existing build script: {script_name}",
                 script_name=existing_script.name,
+                prefix="BUILD",
             )
         else:
             # Generate build scripts using TCLBuilder
-            log_info_safe(logger, "Generating build scripts using TCLBuilder")
+            log_info_safe(
+                logger, "Generating build scripts using TCLBuilder", prefix="BUILD"
+            )
             build_scripts.update(
                 self._generate_build_scripts(board_config, scripts_dir)
             )
@@ -259,7 +273,12 @@ class PCILeechBuildIntegration:
             return scripts
 
         except Exception as e:
-            log_error_safe(logger, "Failed to generate build scripts: {error}", error=e)
+            log_error_safe(
+                logger,
+                "Failed to generate build scripts: {error}",
+                error=e,
+                prefix="BUILD",
+            )
             return {}
 
     def create_unified_build_script(
@@ -312,18 +331,20 @@ puts "Adding source files..."
 """
 
         # Add source files
+        script_content += "\n# Add source files\n"
+        script_content += 'puts "Adding source files..."\n'
         for src_file in build_env["src_files"]:
-            relative_path = src_file.relative_to(build_env["output_dir"])
-            script_content += f'add_files -norecurse "{relative_path}"\n'
+            # Convert to absolute path to avoid path resolution issues in Vivado
+            abs_path = Path(src_file).resolve()
+            script_content += f'add_files -norecurse "{abs_path}"\n'
 
         # Add constraints
         script_content += "\n# Add constraint files\n"
         script_content += 'puts "Adding constraint files..."\n'
         for xdc_file in build_env["xdc_files"]:
-            relative_path = xdc_file.relative_to(build_env["output_dir"])
-            script_content += (
-                f'add_files -fileset constrs_1 -norecurse "{relative_path}"\n'
-            )
+            # Convert to absolute path to avoid path resolution issues in Vivado
+            abs_path = Path(xdc_file).resolve()
+            script_content += f'add_files -fileset constrs_1 -norecurse "{abs_path}"\n'
 
         # Add synthesis and implementation
         script_content += """
@@ -435,12 +456,13 @@ def integrate_pcileech_build(
         )
         if warnings:
             for warning in warnings:
-                log_warning_safe(logger, "{warning}", warning=warning)
+                log_warning_safe(logger, "{warning}", warning=warning, prefix="BUILD")
         if not is_compatible:
             log_error_safe(
                 logger,
                 "Board {board_name} may not be fully compatible with device configuration",
                 board_name=board_name,
+                prefix="BUILD",
             )
 
     return integration.create_unified_build_script(board_name, device_config)
