@@ -13,14 +13,9 @@ from pathlib import Path
 
 from ..string_utils import log_error_safe, log_info_safe
 
-# Add src to path
-sys.path.insert(0, str(Path(__file__).parent.parent))
-
 try:
-    from .vivado_error_reporter import (
-        VivadoErrorReporter,
-        run_vivado_with_error_reporting,
-    )
+    from .vivado_error_reporter import (VivadoErrorReporter,
+                                        run_vivado_with_error_reporting)
     from .vivado_utils import find_vivado_installation
 except ImportError as e:
     print(f"Error importing modules: {e}")
